//
//  ZMSCameraUpdate.h
//  ZDCMaps
//
//  Created by zdc on 2014/12/08.
//  Copyright (c) 2014年 ZENRIN DataCom Co., LTD.. All rights reserved.
//

#ifndef ZDCMaps_ZMSCameraUpdate_h
#define ZDCMaps_ZMSCameraUpdate_h

#import <CoreLocation/CoreLocation.h>
#import <UIKit/UIKit.h>


@class ZMSCameraPosition;
@class ZMSCoordinateBounds;

/**
 * ZMSCameraUpdateはZMSMapViewに更新を適用できるオブジェクト
 *
 * カメラの更新をするためのロジックをカプセル化することができ、以下の
 * ファクトリーメソッドを利用した作成する
 */
@interface ZMSCameraUpdate : NSObject


/**
 * 地図を1レベル拡大するための ZMSCameraUpdate を作成する
  * @return ZMSCameraUpdate
 */
+ (ZMSCameraUpdate *)zoomIn;

/**
 * 地図を1レベル縮小するためのZMSCameraUpdateを作成する
 * @return ZMSCameraUpdate
 */
+ (ZMSCameraUpdate *)zoomOut;

/**
 * 任意のレベル分だけ拡大・縮小するZMSCameraUpdateを作成する
 * @param delta 拡縮したい変更分ズームレベル
 * @return ZMSCameraUpdate
 */
+ (ZMSCameraUpdate *)zoomBy:(float)delta;

/**
 * 任意のレベルへ拡大縮小するZMSCameraUpdateを作成する
 * @param zoom 拡縮したいズームレベル
 * @return ZMSCameraUpdate
 */
+ (ZMSCameraUpdate *)zoomTo:(float)zoom;


/**
 * 任意の緯度経度座標へ移動するZMSCameraUpdateを作成する
 * @param target 緯度経度
 * @return ZMSCameraUpdate
 */
+ (ZMSCameraUpdate *)setTarget:(CLLocationCoordinate2D)target;

/**
 * 任意の緯度経度座標かつ任意のズームレベルへ移動するZMSCameraUpdateを作成する
 * @param target 緯度経度
 * @param zoom ズームレベル
 * @return ZMSCameraUpdate
 */
+ (ZMSCameraUpdate *)setTarget:(CLLocationCoordinate2D)target
                          zoom:(float)zoom;

/**
 * 任意のZMSCameraPositionのZMSCameraUpdateを作成する
 * @param camera カメラオブジェクト
 * @return ZMSCameraUpdate
 */
+ (ZMSCameraUpdate *)setCamera:(ZMSCameraPosition *)camera;

/**
 * 任意の矩形領域が中心で地図ビュー内に最も大きく表示される
 * ZMSCameraUpdateを作成する
 * パディング値は64ポイントがデフォルトで設定されている
 * 返却されるZMSCameraUpdateでは、カメラの回転と傾きは0度になる
 * @param bounds バウンズ（左下・右上座標による矩形領域）
 * @return ZMSCameraUpdate
 */
+ (ZMSCameraUpdate *)fitBounds:(ZMSCoordinateBounds *)bounds;

/** 
 * 任意の矩形領域が中心で地図ビュー内に最も大きく表示される
 * ZMSCameraUpdateを作成する
 * 任意のパディングを設定することができる
 * @param bounds バウンズ（左下・右上座標による矩形領域）
 * @param padding バウンズのパディング範囲(ポイント)
 * @return ZMSCameraUpdate
 */
+ (ZMSCameraUpdate *)fitBounds:(ZMSCoordinateBounds *)bounds
                     withPadding:(CGFloat)padding;

/**
 * 任意の矩形領域が中心で地図ビュー内に最も大きく表示される
 * ZMSCameraUpdateを作成する
 * 任意のUIEdgeInsetsにより上下左右それぞれに任意のパディング
 * 領域を設定することができる
 * @param bounds バウンズ（左下・右上座標による矩形領域）
 * @param edgeInsets 上下左右パディング範囲（ポイント）
 * @return ZMSCameraUpdate
 */
+ (ZMSCameraUpdate *)fitBounds:(ZMSCoordinateBounds *)bounds
               widthEdgeInsets:(UIEdgeInsets)edgeInsets;


/**
 * スクリーン座標系におけるx,y方向の移動量を指定し
 * 地図中心を移動するZMSCameraUpdateを作成する
 * x方向に正の値をセットすると地図は右方向へ移動する
 * y方向に正の値をセットすると地図は下方向へ移動する
 * @param dX X方向への移動量 (ポイント)
 * @param dY Y方向への移動量 (ポイント)
 * @return ZMSCameraUpdate
 */
+ (ZMSCameraUpdate *)scrollByX:(CGFloat)dX
                             Y:(CGFloat)dY;

/**
 * スクリーン座標における位置(CGPoint)の位置を中心として、
 * 指定された縮尺レベル(zoom)に地図を移動する。
 * スクリーン座標は左上が原点
 * @param zoom 拡縮したいズームレベル
 * @param point 拡縮中心のスクリーン座標
 * @return ZMSCameraUpdate
 */
+ (ZMSCameraUpdate *)zoomBy:(float)zoom
                    atPoint:(CGPoint)point;

@end

#endif
